/*
 * GraphElementsTest.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#include "GraphElementsTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "factory/XmlDataFactory.hpp"

#include "graph/GraphClasses.h"


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GraphElementsTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION(GraphElementsTest);


void GraphElementsTest::testCopyConstruct()
{
	// Common
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");

	// DirectedEdge
	graph::DirectedEdge de1(n1, n2);
	graph::DirectedEdge de2(n1, n3);

	graph::DirectedEdge de1_2(de1);
	CPPUNIT_ASSERT(de1_2 == de1);

	graph::DirectedEdge de2_2(de2);
	CPPUNIT_ASSERT(de2_2 == de2);

	graph::DirectedEdge de1_3(std::move(de1));
	CPPUNIT_ASSERT(de1_3 == de1_2);

	graph::DirectedEdge de2_3(std::move(de2));
	CPPUNIT_ASSERT(de2_3 == de2_2);

	// UndirectedEdge
	graph::UndirectedEdge ue1(n1, n2);
	graph::UndirectedEdge ue2(n1, n3);

	graph::UndirectedEdge ue1_2(ue1);
	CPPUNIT_ASSERT(ue1_2 == ue1);

	graph::UndirectedEdge ue2_2(ue2);
	CPPUNIT_ASSERT(ue2_2 == ue2);

	graph::UndirectedEdge ue1_3(std::move(ue1));
	CPPUNIT_ASSERT(ue1_3 == ue1_2);

	graph::UndirectedEdge ue2_3(std::move(ue2));
	CPPUNIT_ASSERT(ue2_3 == ue2_2);

	// Node
	graph::Node n1_2(n1);
	CPPUNIT_ASSERT(n1_2 == n1);

	graph::Node n2_2(n2);
	CPPUNIT_ASSERT(n2_2 == n2);

	graph::Node n3_2(n3);
	CPPUNIT_ASSERT(n3_2 == n3);

	graph::Node n1_3(std::move(n1));
	CPPUNIT_ASSERT(n1_3 == n1_2);

	graph::Node n2_3(std::move(n2));
	CPPUNIT_ASSERT(n2_3 == n2_2);

	graph::Node n3_3(std::move(n3));
	CPPUNIT_ASSERT(n3_3 == n3_2);
}

void GraphElementsTest::testEqual()
{
	// Node
	graph::Node n1("n1");
	graph::Node n1_("n1");
	graph::Node n2("n2");
	graph::Node n2_("n2");
	graph::Node n3("n3");
	graph::Node n3_("n3");

	CPPUNIT_ASSERT(n1 == n1);
	CPPUNIT_ASSERT(n1 == n1_);
	CPPUNIT_ASSERT(n2 == n2);
	CPPUNIT_ASSERT(n2 == n2_);
	CPPUNIT_ASSERT(n3 == n3);
	CPPUNIT_ASSERT(n3 == n3_);

	CPPUNIT_ASSERT(n1 != n2);
	CPPUNIT_ASSERT(n1 != n3);
	CPPUNIT_ASSERT(n2 != n1);
	CPPUNIT_ASSERT(n3 != n1);

	ext::compare<graph::Node> n_cmp;
	CPPUNIT_ASSERT_EQUAL(0, n_cmp(n1, n1_));
	CPPUNIT_ASSERT_EQUAL(0, n_cmp(n1, n1));
	CPPUNIT_ASSERT(n_cmp(n1, n2) != 0);
	CPPUNIT_ASSERT(n_cmp(n1, n3) != 0);
	CPPUNIT_ASSERT(n_cmp(n1, n3_) != 0);

	// DirectedEdge
	graph::DirectedEdge de1(n1, n2);
	graph::DirectedEdge de1_(n1, n2);
	graph::DirectedEdge de2(n1, n3);
	graph::DirectedEdge de2_(n1, n3);
	graph::DirectedEdge de3(n1, n3, "a");
	graph::DirectedEdge de3_(n1, n3, "a");

	CPPUNIT_ASSERT(de1 == de1);
	CPPUNIT_ASSERT(de1 == de1_);
	CPPUNIT_ASSERT(de2 == de2);
	CPPUNIT_ASSERT(de2 == de2_);
	CPPUNIT_ASSERT(de3 == de3);
	CPPUNIT_ASSERT(de3 == de3_);

	CPPUNIT_ASSERT(de1 != de2);
	CPPUNIT_ASSERT(de1 != de3);
	CPPUNIT_ASSERT(de2 != de1);
	CPPUNIT_ASSERT(de3 != de1);

	ext::compare<graph::DirectedEdge> d_cmp;
	CPPUNIT_ASSERT_EQUAL(0, d_cmp(de1, de1_));
	CPPUNIT_ASSERT_EQUAL(0, d_cmp(de1, de1));
	CPPUNIT_ASSERT(d_cmp(de1, de2) != 0);
	CPPUNIT_ASSERT(d_cmp(de1, de3) != 0);
	CPPUNIT_ASSERT(d_cmp(de1, de3_) != 0);

	// UndirectedEdge
	graph::UndirectedEdge ue1(n1, n2);
	graph::UndirectedEdge ue1_(n1, n2);
	graph::UndirectedEdge ue1_s(n2, n1);
	graph::UndirectedEdge ue2(n1, n3);
	graph::UndirectedEdge ue2_(n1, n3);
	graph::UndirectedEdge ue3(n1, n3, "a");
	graph::UndirectedEdge ue3_(n1, n3, "a");

	CPPUNIT_ASSERT(ue1 == ue1);
	CPPUNIT_ASSERT(ue1 == ue1_);
	CPPUNIT_ASSERT(ue1 == ue1_s);
	CPPUNIT_ASSERT(ue1_ == ue1_s);
	CPPUNIT_ASSERT(ue2 == ue2);
	CPPUNIT_ASSERT(ue2 == ue2_);
	CPPUNIT_ASSERT(ue3 == ue3);
	CPPUNIT_ASSERT(ue3 == ue3_);

	CPPUNIT_ASSERT(ue1 != ue2);
	CPPUNIT_ASSERT(ue1 != ue3);
	CPPUNIT_ASSERT(ue2 != ue1);
	CPPUNIT_ASSERT(ue3 != ue1);

	ext::compare<graph::UndirectedEdge> u_cmp;
	CPPUNIT_ASSERT_EQUAL(0, u_cmp(ue1, ue1_));
	CPPUNIT_ASSERT_EQUAL(0, u_cmp(ue1, ue1));
	CPPUNIT_ASSERT_EQUAL(0, u_cmp(ue1, ue1_s));
	CPPUNIT_ASSERT_EQUAL(0, u_cmp(ue1_, ue1_s));
	CPPUNIT_ASSERT(u_cmp(ue1, ue2) != 0);
	CPPUNIT_ASSERT(u_cmp(ue1, ue3) != 0);
	CPPUNIT_ASSERT(u_cmp(ue1, ue3_) != 0);
}

// testLessThan and testHash tests the UndirectedEdge for
// the case when UndirectedEdge(n1, n2) == UndirectedEdge(n2, n1)
// and thus they must compare exactly the same against other edges
// and also have the same hash
void GraphElementsTest::testLessThan()
{
	// Common
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");
	graph::Node n5("n5");

	// UndirectedEdge
	graph::UndirectedEdge ue1(n1, n2);
	graph::UndirectedEdge ue1_(n2, n1);
	graph::UndirectedEdge ue2(n1, n3);
	graph::UndirectedEdge ue3(n1, n3);
	graph::UndirectedEdge ue4(n1, n4);
	graph::UndirectedEdge ue5(n5, n1);

	CPPUNIT_ASSERT_EQUAL(ue1 < ue1_, ue1_ < ue1_);
	CPPUNIT_ASSERT_EQUAL(ue1 < ue2, ue1_ < ue2);
	CPPUNIT_ASSERT_EQUAL(ue1 < ue3, ue1_ < ue3);
	CPPUNIT_ASSERT_EQUAL(ue1 < ue4, ue1_ < ue4);
	CPPUNIT_ASSERT_EQUAL(ue1 < ue5, ue1_ < ue5);
}

void GraphElementsTest::testHash()
{
	// Common
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");
	graph::Node n4("n4");

	// UndirectedEdge
	graph::UndirectedEdge ue1(n1, n2);
	graph::UndirectedEdge ue1_(n2, n1);
	graph::UndirectedEdge ue2(n1, n3);
	graph::UndirectedEdge ue2_(n3, n1);
	graph::UndirectedEdge ue3(n2, n4);
	graph::UndirectedEdge ue3_(n4, n2);

	std::hash<graph::UndirectedEdge> hash_func;
	CPPUNIT_ASSERT_EQUAL(hash_func(ue1), hash_func(ue1_));
	CPPUNIT_ASSERT_EQUAL(hash_func(ue2), hash_func(ue2_));
	CPPUNIT_ASSERT_EQUAL(hash_func(ue3), hash_func(ue3_));
}

